import pythoncom
from win32com.client import Dispatch

from .python_api import reload_module, preload_module, call_function, wrap_python


# Note: change encoding here if needed. Warning: VBA doesn't support UTF-8 encoded modules!
_windows_cyrillic_encoding = 'cp1251'


class ComPython:
    _public_methods_ = ['CallFunction', 'ReloadModule', 'ImportModule', 'WrapPython', 'Eval', 'Exec']

    def CallFunction(self, module_name, func_name: str, args=None):
        """Call python function"""
        unwrapped_args = None if args is None else tuple(_unwrap_com(arg) for arg in args)
        result = call_function(module_name, func_name, unwrapped_args)
        return _wrap_com(result)

    def ReloadModule(self, module_name: str):
        """Reload imported module"""
        return reload_module(module_name)

    def ImportModule(self, module_path: str):
        """Load module"""
        return preload_module(module_path)

    def WrapPython(self, source_module: str, destination_file: str):
        """"Create wrapper for decorated python module functions"""
        return wrap_python(source_module, destination_file, encoding=_windows_cyrillic_encoding)

    def Eval(self, expression: str):
        """Evaluate expression in current context"""
        result = eval(expression)
        return _wrap_com(result)

    def Exec(self, statement: str):
        """Execute statement in current context"""
        exec(statement)


def _unwrap_com(com_value):
    value_type = type(com_value)
    if value_type is pythoncom.TypeIIDs[pythoncom.IID_IDispatch]:
        return Dispatch(com_value)
    return com_value


def _wrap_com(py_value):
    value_type = type(py_value)
    if value_type is tuple:
        return [element for element in py_value]
    elif value_type is dict:
        result = Dispatch('Scripting.Dictionary')
        for (key, value) in py_value.items():
            result.Add(key, value)
        return result
    return py_value
