import logging
import win32api
import win32event
import pythoncom
import pywintypes
import win32com.server.util as server_util
import win32com.server.policy as server_policy

from .comwrappers import ComPython

logger = logging.getLogger(__name__)


def start_server(clsid_str):
    """Start the COM server, clsid_str is the dispatcher object GUID created in DLL module"""
    class_id = pywintypes.IID(clsid_str)
    _override_com_policy(class_id)
    (factory, revokeId) = _register_class_object(class_id)

    pythoncom.EnableQuitMessage(win32api.GetCurrentThreadId())
    pythoncom.CoInitialize()
    pythoncom.CoResumeClassObjects()

    _run_python_loop(class_id)

    pythoncom.CoRevokeClassObject(revokeId)
    pythoncom.CoUninitialize()


def _override_com_policy(class_id):
    class WrapPythonPolicy(server_policy.DefaultPolicy):
        def _CreateInstance_(self, request_clsid, dispatch_id):
            if request_clsid == class_id:
                return server_util.wrap(ComPython(), dispatch_id)
            else:
                return server_policy.DefaultPolicy._CreateInstance_(self, request_clsid, dispatch_id)
    server_policy.DefaultPolicy = WrapPythonPolicy


def _register_class_object(class_id):
    factory = pythoncom.MakePyFactory(class_id)
    revokeId = pythoncom.CoRegisterClassObject(
        class_id, factory,
        pythoncom.CLSCTX_LOCAL_SERVER,
        pythoncom.REGCLS_MULTIPLEUSE | pythoncom.REGCLS_SUSPENDED
    )
    return (factory, revokeId)


def _run_python_loop(class_id):
    msg = f"Python server running, class_id={class_id}"
    if logger.hasHandlers():
        logger.info(msg)
    else:
        print(msg)

    while True:
        rc = win32event.MsgWaitForMultipleObjects(
            (),  # list of objects
            0,   # wait for all = False
            win32event.INFINITE,  # no timeout
            win32event.QS_ALLEVENTS  # accept any input
        )
        if rc == win32event.WAIT_OBJECT_0:
            if pythoncom.PumpWaitingMessages():
                break  # wm_quit
