#include "pch.h"

namespace vbatopy::utils {

std::string GUIDToStr(const GUID guid) {
	return std::format("{{{:08x}-{:04x}-{:04x}-{:02x}{:02x}-{:02x}{:02x}{:02x}{:02x}{:02x}{:02x}}}",
										 guid.Data1, guid.Data2, guid.Data3,
										 guid.Data4[0], guid.Data4[1], 
										 guid.Data4[2], guid.Data4[3], guid.Data4[4], 
										 guid.Data4[5], guid.Data4[6], guid.Data4[7]); // NOLINT(cppcoreguidelines-avoid-magic-numbers, readability-magic-numbers)
}

void ToVariant(const char* str, VARIANT* var) {
	::VariantClear(var);

	int sz = static_cast<int>(strlen(str)) + 1;
	auto wideStr = std::make_unique<OLECHAR[]>(sz);
	::MultiByteToWideChar(CP_ACP, 0, str, static_cast<int>(sz * sizeof(OLECHAR)), wideStr.get(), sz);
	var->vt = VT_BSTR;
	var->bstrVal = SysAllocString(wideStr.get());
}

std::string GetLastErrorMessage() {
	DWORD dwError = ::GetLastError();
	char* buffer{};
	if (0 == ::FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
													nullptr,	dwError, 0, reinterpret_cast<LPSTR>(&buffer),	0, nullptr)) {
		return "Could not get error message: FormatMessage failed.";
	}
	auto errorMessage = std::string{ buffer };
	::LocalFree(buffer);
	return errorMessage;
}

void ApplyPythonEscaping(std::string& target) {
	std::string result{};
	result.reserve(size(target));
	for (const auto& c : target) {
		if (c == '\\') {
			result += R"(\\)";
		} else if (c == '&') {
			result += "^&";
		} else {
			result += c;
		}
	}
	if (size(target) != size(result)) {
		target = result;
	}
}

std::string GeneratePythonCommand(std::string pythonExe, std::string sources, const GUID clsid) {
	ApplyPythonEscaping(sources);
	return std::format(R"(cmd.exe /C {} -B -c )"
										 R"("import sys, os; sys.path[0:0]=os.path.normcase(os.path.expandvars(\"{}\")).split(';'); )"
										 R"(import vbatopy; vbatopy.server.start_server('{}') ")", pythonExe, sources, GUIDToStr(clsid));
}

} // namespace vbatopy::utils