#include "pch.h"

using vbatopy::Server;

// DLL entry point
BOOL WINAPI DllMain(HINSTANCE /*hinstDLL*/, DWORD fdwReason, LPVOID /*lpvReserved*/) {
	switch (fdwReason) {
	case DLL_PROCESS_ATTACH:
		return TRUE;

	case DLL_PROCESS_DETACH:
		Server::Instance().ForceKill();
		return TRUE;

	default:
		return TRUE;
	}
}

DLLEXPORT HRESULT __stdcall GetServer(VARIANT* pyResult) {
	try {
		::VariantClear(pyResult);
		auto& server = Server::Instance();

		if (server.CheckStatus()) {
			pyResult->vt = VT_DISPATCH;
			server.dispatcher->QueryInterface<IDispatch>(&pyResult->pdispVal);
			return S_OK;
		} else {
			pyResult->vt = VT_DISPATCH;
			pyResult->pdispVal = nullptr;
			return S_OK;
		}
	} catch (const std::exception& e) {
		vbatopy::utils::ToVariant(e.what(), pyResult);
		return E_FAIL;
	}
}

DLLEXPORT HRESULT __stdcall StartServer(VARIANT* pyResult, const char* command, int showConsole) {
	try {
		::VariantClear(pyResult);

		auto& server = Server::Instance();
		if (server.CheckStatus()) {
			throw std::runtime_error("Server already running");
		}

		server.Setup(command, showConsole != 0);
		server.Start();

		pyResult->vt = VT_DISPATCH;
		server.dispatcher->QueryInterface<IDispatch>(&pyResult->pdispVal);

		return S_OK;
	} catch (const std::exception& e) {
		vbatopy::utils::ToVariant(e.what(), pyResult);
		return E_FAIL;
	}
}

DLLEXPORT HRESULT __stdcall StartPythonServer(VARIANT* pyResult, const char* python, const char* sources, int showConsole) {
	try {
		::VariantClear(pyResult);
		if (python == nullptr || strlen(python) == 0) {
			throw std::invalid_argument("Python command missing");
		}

		auto& server = Server::Instance();
		if (server.CheckStatus()) {
			throw std::runtime_error("Server already running");
		}

		server.Setup(vbatopy::utils::GeneratePythonCommand(python, sources, server.clsid), showConsole != 0);
		server.Start();

		pyResult->vt = VT_DISPATCH;
		server.dispatcher->QueryInterface<IDispatch>(&pyResult->pdispVal);

		return S_OK;
	} catch (const std::exception& e) {
		vbatopy::utils::ToVariant(e.what(), pyResult);
		return E_FAIL;
	}
}

DLLEXPORT HRESULT __stdcall KillServer() {
	try {
		auto& server = Server::Instance();
		server.Kill();
		return S_OK;
	} catch (const std::exception& /*e*/) {
		return E_FAIL;
	}
}